import TestDefinitionActions from '../TestDefinition.actions';
import {store} from '../../store';
import {HTTP_METHOD} from '../../../constants/Common.constants';
import TestDefinitionSelectors from '../../../selectors/TestDefinition.selectors';
import TestRunMock from '../../../models/__mocks__/TestRun.mock';

jest.mock('../../../selectors/TestDefinition.selectors', () => ({
  selectDefinitionList: jest.fn(),
}));

const selectTestMock = TestDefinitionSelectors.selectDefinitionList as unknown as jest.Mock;

describe('TestDefinitionActions', () => {
  beforeEach(() => {
    fetchMock.resetMocks();
  });

  describe('publish', () => {
    it('should trigger the set definition and rerun requests', async () => {
      selectTestMock.mockImplementationOnce(() => []);

      fetchMock.mockResponseOnce(JSON.stringify({}));
      fetchMock.mockResponseOnce(JSON.stringify(TestRunMock.raw()));

      await store.dispatch(
        TestDefinitionActions.publish({
          testId: 'testId',
          runId: 'runId',
        })
      );

      const setRequest = fetchMock.mock.calls[0][0] as Request;
      const reRunRequest = fetchMock.mock.calls[1][0] as Request;

      expect(setRequest.url).toEqual('http://localhost/api/tests/testId/definition');
      expect(setRequest.method).toEqual(HTTP_METHOD.PUT);

      expect(reRunRequest.url).toEqual('http://localhost/api/tests/testId/run/runId/rerun');
      expect(reRunRequest.method).toEqual(HTTP_METHOD.POST);

      expect(fetchMock.mock.calls.length).toBe(2);
    });
  });

  describe('dry run', () => {
    it('should trigger the dry run request', async () => {
      selectTestMock.mockImplementationOnce(() => []);

      fetchMock.mockResponseOnce(JSON.stringify(TestRunMock.raw()));
      await store.dispatch(
        TestDefinitionActions.dryRun({
          testId: 'testId',
          runId: 'runId',
          definitionList: [],
        })
      );

      const dryRunRequest = fetchMock.mock.calls[0][0] as Request;

      expect(dryRunRequest.url).toEqual('http://localhost/api/tests/testId/run/runId/dry-run');
      expect(dryRunRequest.method).toEqual(HTTP_METHOD.PUT);

      expect(fetchMock.mock.calls.length).toBe(1);
    });
  });
});
